@@ -1,4 +1,5 @@
 """Classes and function useful for trying out mincepy functionality"""
+import os
 import random
 import uuid
 
@@ -8,12 +9,22 @@ import pytest
 
 import mincepy
 
+ENV_ARCHIVE_URI = 'MINCEPY_TEST_URI'
+DEFAULT_ARCHIVE_URI = 'mongodb://localhost/mincepy-tests'
+
+# pylint: disable=redefined-outer-name
+
 
 @pytest.fixture
-def mongodb_archive():
-    client = pymongo.MongoClient()
-    client.drop_database('test_database')
-    db = client.test_database
+def archive_uri():
+    return os.environ.get(ENV_ARCHIVE_URI, DEFAULT_ARCHIVE_URI)
+
+
+@pytest.fixture
+def mongodb_archive(archive_uri):
+    client = pymongo.MongoClient(archive_uri)
+    db = client.get_default_database()  # pylint: disable=invalid-name
+    client.drop_database(db)
     mongo_archive = mincepy.mongo.MongoArchive(db)
     yield mongo_archive
     client.drop_database(db)
@@ -46,18 +57,20 @@ class Garage(mincepy.SimpleSavable):
         super(Garage, self).__init__()
         self.car = car
 
+    def save_instance_state(self, saver: mincepy.Saver):
+        state = super().save_instance_state(saver)
+        return state
+
 
 class Person(mincepy.SimpleSavable):
     TYPE_ID = uuid.UUID('d60ca740-9fa6-4002-83f6-e4c91403e41b')
-    ATTRS = 'name', 'age'
+    ATTRS = 'name', 'age', mincepy.AsRef('car')
 
-    def __init__(self, name, age):
+    def __init__(self, name, age, car=None):
         super(Person, self).__init__()
         self.name = name
         self.age = age
-
-    def __eq__(self, other):
-        return self.name == other.name and self.age == other.age
+        self.car = car
 
 
 class Cycle(mincepy.SimpleSavable):
